@@ -1,6 +1,3 @@
-#define _POSIX_C_SOURCE 200809L
-#include <assert.h>
-#include <errno.h>
 #include <getopt.h>
 #include <libgen.h>
 #include <stdlib.h>
@@ -11,6 +8,7 @@
 
 #include "config.h"
 #include "criteria.h"
+#include "string-util.h"
 #include "types.h"
 
 static int32_t max(int32_t a, int32_t b) {
@@ -126,7 +124,8 @@ void init_default_style(struct mako_style *style) {
 	style->anchor =
 		ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP | ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT;
 
-	style->button_bindings.left.action = MAKO_BINDING_INVOKE_DEFAULT_ACTION;
+	style->button_bindings.left.action = MAKO_BINDING_INVOKE_ACTION;
+	style->button_bindings.left.action_name = strdup(DEFAULT_ACTION_KEY);
 	style->button_bindings.right.action = MAKO_BINDING_DISMISS;
 	style->button_bindings.middle.action = MAKO_BINDING_NONE;
 	style->touch_binding.action = MAKO_BINDING_DISMISS;
@@ -141,6 +140,7 @@ void init_empty_style(struct mako_style *style) {
 
 static void finish_binding(struct mako_binding *binding) {
 	free(binding->command);
+	free(binding->action_name);
 }
 
 void finish_style(struct mako_style *style) {
@@ -163,6 +163,9 @@ static void copy_binding(struct mako_binding *dst,
 	if (src->command != NULL) {
 		dst->command = strdup(src->command);
 	}
+	if (src->action_name != NULL) {
+		dst->action_name = strdup(src->action_name);
+	}
 }
 
 // Update `target` with the values specified in `style`. If a failure occurs,
@@ -641,12 +644,18 @@ static bool apply_style_option(struct mako_style *style, const char *name,
 			binding.action = MAKO_BINDING_NONE;
 		} else if (strcmp(value, "dismiss") == 0) {
 			binding.action = MAKO_BINDING_DISMISS;
+		} else if (strcmp(value, "dismiss --no-history") == 0) {
+			binding.action = MAKO_BINDING_DISMISS_NO_HISTORY;
 		} else if (strcmp(value, "dismiss-all") == 0) {
 			binding.action = MAKO_BINDING_DISMISS_ALL;
 		} else if (strcmp(value, "dismiss-group") == 0) {
 			binding.action = MAKO_BINDING_DISMISS_GROUP;
 		} else if (strcmp(value, "invoke-default-action") == 0) {
-			binding.action = MAKO_BINDING_INVOKE_DEFAULT_ACTION;
+			binding.action = MAKO_BINDING_INVOKE_ACTION;
+			binding.action_name = strdup(DEFAULT_ACTION_KEY);
+		} else if (has_prefix(value, "invoke-action ")) {
+			binding.action = MAKO_BINDING_INVOKE_ACTION;
+			binding.action_name = strdup(value + strlen("invoke-action "));
 		} else if (has_prefix(value, "exec ")) {
 			binding.action = MAKO_BINDING_EXEC;
 			binding.command = strdup(value + strlen("exec "));
@@ -691,35 +700,40 @@ static bool file_exists(const char *path) {
 }
 
 static char *get_config_path(void) {
-	static const char *config_paths[] = {
-		"$HOME/.mako/config",
-		"$XDG_CONFIG_HOME/mako/config",
+	const char *home = getenv("HOME");
+	if (home == NULL) {
+		fprintf(stderr, "HOME env var not set\n");
+		return NULL;
+	}
+
+	const char *config_home = getenv("XDG_CONFIG_HOME");
+	char *config_home_fallback = NULL;
+	if (config_home == NULL || config_home[0] == '\0') {
+		config_home_fallback = mako_asprintf("%s/.config", home);
+		config_home = config_home_fallback;
+	}
+
+	char *config_paths[] = {
+		mako_asprintf("%s/.mako/config", home),
+		mako_asprintf("%s/mako/config", config_home),
 	};
 
-	if (!getenv("XDG_CONFIG_HOME")) {
-		char *home = getenv("HOME");
-		if (!home) {
-			return NULL;
-		}
-		char config_home[strlen(home) + strlen("/.config") + 1];
-		strcpy(config_home, home);
-		strcat(config_home, "/.config");
-		setenv("XDG_CONFIG_HOME", config_home, 1);
-	}
-
-	for (size_t i = 0; i < sizeof(config_paths) / sizeof(char *); ++i) {
-		wordexp_t p;
-		if (wordexp(config_paths[i], &p, 0) == 0) {
-			char *path = strdup(p.we_wordv[0]);
-			wordfree(&p);
-			if (file_exists(path)) {
-				return path;
-			}
-			free(path);
+	size_t config_paths_len = sizeof(config_paths) / sizeof(config_paths[0]);
+	char *found_path = NULL;
+	for (size_t i = 0; i < config_paths_len; ++i) {
+		char *path = config_paths[i];
+		if (file_exists(path)) {
+			found_path = strdup(path);
+			break;
 		}
 	}
 
-	return NULL;
+	for (size_t i = 0; i < config_paths_len; ++i) {
+		free(config_paths[i]);
+	}
+	free(config_home_fallback);
+
+	return found_path;
 }
 
 int load_config_file(struct mako_config *config, char *config_arg) {
