@@ -2,11 +2,11 @@
 #define MAKO_DBUS_H
 
 #include <stdbool.h>
-#ifdef HAVE_LIBSYSTEMD
+#if defined(HAVE_LIBSYSTEMD)
 #include <systemd/sd-bus.h>
-#elif HAVE_LIBELOGIND
+#elif defined(HAVE_LIBELOGIND)
 #include <elogind/sd-bus.h>
-#elif HAVE_BASU
+#elif defined(HAVE_BASU)
 #include <basu/sd-bus.h>
 #endif
 
