@@ -5,11 +5,11 @@
 #include <stdbool.h>
 #include <time.h>
 #include <wayland-client.h>
-#ifdef HAVE_LIBSYSTEMD
+#if defined(HAVE_LIBSYSTEMD)
 #include <systemd/sd-bus.h>
-#elif HAVE_LIBELOGIND
+#elif defined(HAVE_LIBELOGIND)
 #include <elogind/sd-bus.h>
-#elif HAVE_BASU
+#elif defined(HAVE_BASU)
 #include <basu/sd-bus.h>
 #endif
 
