@@ -1,8 +1,6 @@
-#define _XOPEN_SOURCE 700
 #include <stdlib.h>
 #include <cairo/cairo.h>
 #include <pango/pangocairo.h>
-#include <math.h>
 
 #include "config.h"
 #include "criteria.h"
@@ -13,6 +11,8 @@
 #include "wlr-layer-shell-unstable-v1-client-protocol.h"
 #include "icon.h"
 
+#define M_PI 3.14159265358979323846
+
 // HiDPI conventions: local variables are in surface-local coordinates, unless
 // they have a "buffer_" prefix, in which case they are in buffer-local
 // coordinates.
@@ -271,30 +271,32 @@ static int render_notification(cairo_t *cairo, struct mako_state *state, struct
 		switch (style->icon_location) {
 		case MAKO_ICON_LOCATION_LEFT:
 			xpos = offset_x + style->border_size +
-				(text_x - icon->width) / 2;
+				style->padding.left;
 			ypos = ypos_center;
 			break;
 		case MAKO_ICON_LOCATION_RIGHT:
 			xpos = offset_x + notif_width - style->border_size -
-				icon->width - style->margin.right;
+				style->padding.right - icon->width;
 			ypos = ypos_center;
 			break;
 		case MAKO_ICON_LOCATION_TOP:
 			xpos = xpos_center;
-			ypos = offset_y + style->border_size;
+			ypos = offset_y + style->border_size +
+				style->padding.top;
 			break;
 		case MAKO_ICON_LOCATION_BOTTOM:
 			xpos = xpos_center;
-			ypos = offset_y + text_y + text_height + style->margin.bottom;
+			ypos = offset_y + notif_height - style->border_size -
+				style->padding.bottom - icon->height;
 			break;
 		}
 		draw_icon(cairo, icon, xpos, ypos, scale);
 	}
 
 	if (icon_vertical) {
-		text_x = (notif_width - text_width) / 2;
+		text_x = (notif_width - text_width - border_size) / 2;
 	} else {
-		text_y = (notif_height - text_height) / 2;
+		text_y = (notif_height - text_height - border_size) / 2;
 	}
 
 	// Render text
@@ -319,12 +321,15 @@ static int render_notification(cairo_t *cairo, struct mako_state *state, struct
 	return notif_height;
 }
 
-int render(struct mako_surface *surface, struct pool_buffer *buffer, int scale) {
+void render(struct mako_surface *surface, struct pool_buffer *buffer, int scale,
+		int *rendered_width, int *rendered_height) {
 	struct mako_state *state = surface->state;
 	cairo_t *cairo = buffer->cairo;
 
+	*rendered_width = *rendered_height = 0;
+
 	if (wl_list_empty(&state->notifications)) {
-		return 0;
+		return;
 	}
 
 	// Clear
@@ -337,6 +342,7 @@ int render(struct mako_surface *surface, struct pool_buffer *buffer, int scale)
 	size_t visible_count = 0;
 	size_t hidden_count = 0;
 	int total_height = 0;
+	int max_width = 0;
 	int pending_bottom_margin = 0;
 	struct mako_notification *notif;
 	size_t total_notifications = 0;
@@ -399,7 +405,13 @@ int render(struct mako_surface *surface, struct pool_buffer *buffer, int scale)
 			&notif->hotspot, notif->progress);
 		free(text);
 
+		int notif_width =
+			style->width + style->margin.left + style->margin.right;
+
 		total_height += notif_height;
+		if (max_width < notif_width) {
+			max_width = notif_width;
+		}
 		pending_bottom_margin = style->margin.bottom;
 
 		if (notif->group_index < 1) {
@@ -409,7 +421,6 @@ int render(struct mako_surface *surface, struct pool_buffer *buffer, int scale)
 			// single entity for this purpose.
 			++visible_count;
 		}
-
 	}
 
 	if (hidden_count > 0) {
@@ -436,7 +447,7 @@ int render(struct mako_surface *surface, struct pool_buffer *buffer, int scale)
 		char *text = malloc(text_ln + 1);
 		if (text == NULL) {
 			fprintf(stderr, "allocation failed");
-			return 0;
+			return;
 		}
 
 		format_text(style->format, text, format_hidden_text, &data);
@@ -444,11 +455,12 @@ int render(struct mako_surface *surface, struct pool_buffer *buffer, int scale)
 		int hidden_height = render_notification(
 			cairo, state, surface, style, text, NULL, total_height, scale, NULL, 0);
 		free(text);
-		destroy_notification(hidden_notif);
 
 		total_height += hidden_height;
 		pending_bottom_margin = style->margin.bottom;
+		destroy_notification(hidden_notif);
 	}
 
-	return total_height + pending_bottom_margin;
+	*rendered_width = max_width;
+	*rendered_height = total_height;
 }
